# -*- coding: utf-8 -*-
###########################################################################
#
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
#
# librairie pour la ompilation des paquets Eole
#
###########################################################################

from eolepack.configserver import EOLEPACK_FILE, COMPIL_PATH, SVN_ORIG_PATH, DEBEMAIL, DEBFULLNAME, CLONE_DIR
import os, traceback
from os.path import join, isdir, isfile
from shutil import copyfile, rmtree
from glob import glob
import subprocess
from eolepack import eolepackgit, eolepackdb
from eolepack.errors import *

class Build:
    """actions de compilation"""

    def __init__(self, git_project, project, branch=None, distribution='', revision='', rep=''):
        """
        vars: variables des différents fichiers de config
        """
        self.project = project
        self.build_dir = COMPIL_PATH
        if distribution != '':
            self.dist_branch = eolepackdb.get_branches_from_eoledistrib(distribution)
        elif branch in ('None', None):
            # pas de distribution ni de branche spécifiées, on prend master par défaut
            self.dist_branch = 'master'
        if branch != None:
            self.repository, self.branch = branch.split('/')
            self.remote_branch = branch
        else:
            self.repository = 'origin'
            self.branch = self.dist_branch
            self.remote_branch = 'origin/%s' % self.dist_branch

        self.distribution = distribution
        self.revision = revision
        self.rep = rep
        self.git_project = git_project
        self.git_path = join(CLONE_DIR, git_project)
        self.check_up_to_date()

    def check_clone(self):
        """ vérifie si le clone du projet existe, ainsi que la branche upstream
        """
        if not os.path.isdir(join(self.git_path, '.git')):
            if os.path.isdir(self.git_path):
                rmtree(self.git_path)
            try:
                eolepackgit.git_clone(self.git_project)
            except:
                print "!! Erreur clone : "
                traceback.print_exc()
                raise BuildError, 'Erreur de création de clonage du projet %s' % self.git_project
        try:
            #Creation d'une branche a partir d'un tag
            if self.revision not in ('', None):
                eolepackgit.git_checkout_from_tag(self.git_project, self.revision, self.branch)
        except:
            traceback.print_exc()
            raise BuildError, "Erreur lors du checkout de la branche %s" % self.branch

        branches = eolepackgit.git_branch(self.git_project, remote=False)
        try:
            # vérification de la branche upstream
            if 'upstream' not in branches:
                print "Ajout de la branche upstream pour ", self.git_project
                eolepackgit.git_new_branch(self.git_project, 'upstream')
        except:
            traceback.print_exc()
            raise BuildError, "Erreur à l'ajout de la branche upstream"
        try:
            # vérification de la présence de la branche demandée dans le dépot local
            #if self.branch not in branches and self.revision in ('', None):
            #    # checkout de la branche distante avec l'option --track
            #    eolepackgit.git_track_branch(self.git_project, self.branch, self.remote_branch)
            eolepackgit.git_checkout(self.git_project, self.branch)
        except:
            traceback.print_exc()
            raise BuildError, "Erreur lors du checkout de la branche %s" % self.branch

    def check_up_to_date(self):
        """ vérifie que le dépot local du projet est à jour
        """
        self.check_clone()
        eolepackgit.git_pull(self.git_path)

    def insert_debchange(self, long_release, user, autoinc):
        inc = ""
        snap = ""
        if not isfile('debian/changelog'):
            raise ChangelogNotFound
        environ = {}
        environ['DEBEMAIL'] = DEBEMAIL
        environ['DEBFULLNAME'] = DEBFULLNAME
        if self.distribution.endswith("-proposed") or self.distribution.endswith("-dev"):
            release = "-S"
            snap = True
        else:
            release = "-R"
            snap = False
        if not autoinc:
            inc = "-N %s" % long_release
        retcode = subprocess.call('cd %s && DEBEMAIL="eole@ac-dijon.fr" DEBFULLNAME="Equipe Eole" /usr/bin/git-dch -a %s %s --distrib="%s" --debian-branch="%s"' \
                % (self.git_path, release, inc, self.distribution, self.branch), shell=True, env=environ)
        if retcode != 0:
            raise DebChangeError
        if snap:
            compil_msg = "## Nouveau paquet compilé par %s : %s (%s.snapshot) %s depuis %s"% (user, self.project, long_release, self.distribution, self.revision or self.branch or self.dist_branch)
        else:
            compil_msg = "## Nouveau paquet compilé par %s : %s (%s) %s depuis %s"% (user, self.project, long_release, self.distribution, self.revision or self.branch or self.dist_branch)
        try:
            eolepackgit.git_commit(compil_msg, self.git_path)
            #supression des tags rc/dev existants
            if self.distribution.endswith("-proposed") or self.distribution.endswith("-dev"):
                #tag en version~rc/dev
                print "pas de tag"
                #self.tag_clean(long_release)
                #self.tag_build(long_release + '.snapshot')
            else:
                self.tag_build(long_release)
        except:
            traceback.print_exc()
            raise BuildError, "Erreur d'envoi de commit du changelog dans la branche %s" % self.branch

    def debuild(self, user='', passwd=''):
        """ options
        --git-export=<commit-id>
        --git-upstream-branch=branch_name
        --git-debian-branch=branch_name
        --git-no-create-orig
        """
        options = '--git-export-dir=%s' % join(self.build_dir, self.git_project)
        options += ' --git-upstream-branch=%s --git-debian-branch=%s' % (self.branch, self.branch)
        #if self.revision:
        #    options += ' --git-upstream-branch=%s --git-debian-branch=%s --git-export=%s' % (self.revision, self.revision, self.revision)
        #else:
        #    options += ' --git-upstream-branch=%s --git-debian-branch=%s' % (self.branch, self.branch)
        posttag_options = ""
        #if user and passwd:
        #    posttag_options += """--git-posttag="git push --all && git push --tags" """
        build_cmd = "rm -rf %s && cd %s && /usr/bin/git-buildpackage %s | tee -a %s/rapport_%s.txt" \
% (join(COMPIL_PATH, self.git_project, '*'), self.git_path, options, self.build_dir, self.project)
        try:
            print "APPEL BUILD : ", build_cmd
            # purge du fichier de rapport
            open(join(COMPIL_PATH, "rapport_%s.txt" % self.project), "w").close()
            retcode = subprocess.call(build_cmd, shell=True)
            assert retcode == 0
        except:
            print "Erreur à l'exécution de git-buildpackage"
            if glob('%s/*.deb' % join(self.build_dir, self.git_project)) == []:
                # aucun paquet géné
                print "Paquet non retrouvé dans %s" % join(self.build_dir, self.git_project)
            raise BuildError

    def put_deb_repository(self, repository='eole'):
        """
        envoi des paquets sur le repository
        """
        print "Envoi des paquets (dput)"
        return os.system("""cd %s && dput %s *.changes"""% (join(self.build_dir, self.git_project), repository))

    def tag_build(self, long_release):
        eolepackgit.git_buildtag(self.git_path, long_release)
        traceback.print_exc()

    def tag_clean(self, long_release):
        tags = eolepackgit.git_tag(self.git_path)
        if long_release+".snapshot" in [tag.name for tag in tags]:
            eolepackgit.git_del_tag(self.git_path, "debian/"+long_release+".snapshot")

    def push_changes(self, user, long_release):
        """commit des modifications et envoi sur le serveur git"""
        # envoi des modifications sur le dépot
        try:
            eolepackgit.git_push(self.git_path, '--all')
        except:
            traceback.print_exc()
            raise BuildError, "Erreur d'envoi du changelog sur le serveur git"
        try:
            eolepackgit.git_push(self.git_path, '--tags')
        except:
            traceback.print_exc()
            raise BuildError, "Erreur d'envoi des tags sur le serveur git"
