# -*- coding: utf-8 -*-
###########################################################################
#
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
#
# librairie pour la ompilation des paquets Eole
#
###########################################################################

from eolepack.configserver import EOLEPACK_FILE, COMPIL_PATH, SVN_ORIG_PATH, DEBEMAIL, DEBFULLNAME
import os
from os.path import join, isdir, isfile
from shutil import copyfile
from glob import glob
import pysvn
import subprocess
from eolepack import eolepackdb
from eolepack import eolepacksvn
from eolepack.errors import *

class Build:
    """actions de compilation"""
    def __init__(self, id_paquet, paquet, version, distribution, user, password,
                 basedir='trunk', rep='', cfg=None, revision=''):
        """
        vars: variables des différents fichiers de config
        """
        self.project = paquet
        self.version = version
        self.basedir = basedir
        self.revision = revision
        self.rep = rep
        self.cfg = cfg

        self.project_dir = "%s-%s" % (self.project, self.version)
        self.svn_path = join(SVN_ORIG_PATH, eolepackdb.get_svn_path(id_paquet))
        self.export_path = join(self.svn_path, self.basedir, self.rep)

        if self.basedir == 'trunk':
            self.depuis = "(depuis trunk)"
        else:
            self.depuis = "(depuis %s %s)" % (self.basedir, self.rep)
        # init svn + config perso
        self.user = user
        self.password = password
        self.test = 0
        self.svn_client = eolepacksvn.svnclient(self.user, self.password)
        #self.svn_client = pysvn.Client('/home/eole/.eolepacksvn')
        #def login(*args):
        #    self.test += 1
        #    if self.test > 10:
        #        raise WrongPasswd("Erreur d'authentification SVN")
        #    print "Utilisation du compte %s" % self.user
        #    return True, self.user, self.password, False
        #self.svn_client.callback_get_login = login
        print self.export_path

    def svn_export(self):
        """exportation d'un projet
        basedir: renseigne 'trunk', 'tags', 'branches'
        """
        try:
            if self.revision == '':
                self.svn_client.export(self.export_path, self.project_dir)
            else:
                revnum = pysvn.Revision( pysvn.opt_revision_kind.number, int(self.revision) )
                self.svn_client.export(self.export_path, self.project_dir, revision=revnum)
        except pysvn.ClientError, err:
            raise SvnExportError, err

        if not isdir(self.project_dir):
            raise SvnExportError, "L'export n'a pas créé le répertoire %s"%self.project_dir

    def move_to_old(self):
        if isdir(self.project_dir):
            os.system("rm -rf %s"%self.project_dir)
        if not isdir("old"):
            os.mkdir("old")
        os.system("mv *.build *.changes *.deb *.udeb *.dsc *.diff.gz old 2>/dev/null")

    def insert_debchange(self, distribution, long_release, comment):
        if not isfile('debian/changelog'):
            raise ChangelogNotFound
        environ = {}
        environ['DEBEMAIL'] = DEBEMAIL
        environ['DEBFULLNAME'] = DEBFULLNAME
        if comment is None: comment = "no comment"
        comment = comment.split(';')
        # FIXME : bug si accents ???
        retcode = subprocess.call('echo "\n" | debchange -b --distribution="%s" -v %s "%s"' % (distribution, long_release, comment[0]), shell=True, env=environ)
        if retcode != 0:
            raise DebChangeError
        for comm in comment[1:]:
            retcode = subprocess.call('debchange -a "%s"' % comm, shell=True, env=environ)

    def create_tag(self, release):
        def get_log_message():
            # récup des variables parents !
            return True, "** Creation du tag eole%d dans la branche tags de %s %s**"%(release, self.project, self.depuis)
        self.svn_client.callback_get_log_message = get_log_message
        try:
            print self.svn_client.copy(self.export_path, join(self.svn_path, 'tags', 'eole%d')%release)
        except:
            # FIXME
            print "Tag eole%d déjà présent !" % release

    def make_tarfile(self):
        destination = "%s_%s.orig.tar.gz"%(self.project, self.version)
#        try:
#            # FIXME : exclude=['debian'] pas encore dispo !
#            out = tarfile.TarFile.open(destination, 'w:gz')
#            out.add(project_dir, arcname=os.path.basename(project_dir), exclude=['debian'])
#            out.close()
#        except (tarfile.TarError, tarfile.CompressionError, tarfile.StreamError), \
#            err: print "%s" % err
        retcode = subprocess.call("tar --exclude=debian -czf %s %s"%(destination, self.project_dir), shell=True)
        if retcode != 0:
            raise TarFileError

    def debuild(self):
        retcode = subprocess.call("debuild -us -uc -sa | tee -a ../rapport.txt", shell=True)
        print "retcode build : %s" % retcode
        if retcode != 0:
            raise BuildError
        if glob('../*.deb') == []:
            # aucun paquet généré
            raise BuildError

    def sign_packages(self):
        # FIXME
        os.system("""debsign -kBE7294AB *.changes""")

    def create_svn_structure(self):
        """
        vérification/création de la structure svn (tags,...)
        """
        rep_list = [join(self.svn_path, 'branches'), join(self.svn_path, 'tags'), join(self.svn_path, 'trunk')]
        try:
            entry_list = self.svn_client.ls(self.svn_path)
            names = [i.data['name']for i in entry_list]
            for i in rep_list:
                if i not in names:
                    self.svn_client.mkdir(i, 'Creation de la structure du projet %s'%self.project)
        except pysvn.ClientError, err: print "%s" % err

    def prepare_changelog(self):
        """
        récupération du changelog du TRUNK
        cf. commit_changelog
        """
        print self.svn_client.checkout(join(self.svn_path, 'trunk','debian'), 'svn_deb')
        copyfile('svn_deb/changelog', join(self.project_dir, 'debian', 'changelog'))
        #copyfile(join(self.project_dir, 'debian', 'changelog'), 'svn_deb/changelog')

    def commit_changelog(self, project, long_release, distribution):
        """
        commit du changelog (dans le trunk, pour l'instant)
        """
        #print self.svn_client.checkout(join(self.svn_path, 'trunk','debian'), 'svn_deb')
        copyfile(join(self.project_dir, 'debian', 'changelog'), 'svn_deb/changelog')
        olddir= os.getcwd()
        os.chdir("svn_deb")
        print self.svn_client.checkin(['.'], "## Nouveau paquet : %s (%s) %s %s ##"%(project, long_release, distribution, self.depuis))
        os.chdir(olddir)
        os.system("rm -rf svn_deb")

    def put_deb_repository(self, repository='eole'):
        """
        envoi des paquets sur le repository
        """
        #try:
        #    dputfile = open('.dput.cf')
        #    dputfile.close()
        print "envoi des paquets"
        os.system("""dput %s *.changes"""% (repository))
        #except IOError:
        #    print "Fichier %s non trouvé" % dputfile

