# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# eolepackdb : librairie d'accès à la base de données
#
###########################################################################

from eolepack import configserver
from eolepack.errors import *
import psycopg2 as pgsql
import md5

def connect_db():
    # FIXME : blocage si db inaccessible
    try:
        cx = pgsql.connect(database=configserver.DB_NAME, user=configserver.DB_USER,
                           password=configserver.DB_PWD, host=configserver.DB_HOST)
        return cx
    except OperatinalError, error_message:
        return error_message

def execute(query):
    cx = connect_db()
    cu = cx.cursor()
    # FIXME recuperer le nouvel id
    cu.execute(query)
    cu.close()
    cx.commit()
    cx.close()

def select_one(query):
    cx = connect_db()
    cu = cx.cursor()
    cu.execute(query)
    res = cu.fetchone()
    cu.close()
    cx.close()
    return res

def select_all(query):
    cx = connect_db()
    cu = cx.cursor()
    cu.execute(query)
    res = cu.fetchall()
    cu.close()
    cx.close()
    return res

def get_id_paquet(paquet, version, dist_ubuntu):
    """
    renvoit l'id d'un paquet
    """
    query = """select paquets.id from paquets, dist_ubuntu
            where paquets.name='%s'
            and paquets.version='%s'
            and dist_ubuntu.name='%s'
            and dist_ubuntu.id=paquets.id_dist_ubuntu""" % (paquet, version, dist_ubuntu)
    res = select_one(query)
    if res == None:
        raise UnknownPackage
    return res[0]


def get_svn_path(id_paquet):
    query = """select paquets_conf.svn_path
            from paquets_conf
            where paquets_conf.id_paquet=%d;""" % id_paquet
    res = select_one(query)
    if res == None:
        raise UnknownConfig
    return res[0]

def get_package_name(id_paquet):
    query = """ select name from paquets where paquets.id=%d """ % id_paquet
    res = select_one(query)
    if res == None:
        raise UnknownPackage
    return res[0]

def get_last_release(id_paquet):
    query = """select MAX(paquets_release.release)
                from paquets_release
                where paquets_release.id_paquet=%d""" % id_paquet
    res = select_one(query)
    if res[0] == None:
        # le count répond toujours
        raise LastReleaseNotFound
    return res[0]

def get_modules():
    """
    renvoit la liste des modules
    """
    query = """ select modules.id, modules.name from modules; """
    return select_all(query)

def get_id_dist_ubuntu(dist_ubuntu='feisty'):
    """
    renvoit l'id de la distribution
    """
    query = """select id from dist_ubuntu where name='%s';""" % dist_ubuntu
    return select_one(query)[0]

def get_server(dist_ubuntu):
    """
    renvoit l'adresse du serveur de compilation pour la distribution
    """
    query = """select serveur from dist_ubuntu where name='%s';""" % dist_ubuntu
    return select_one(query)[0]

def get_ubuntu_from_dist(longdist):
    """ exemple : longdist = eole-2.1-dev """
    query = """select dist_ubuntu from dist_eole where name='%s';""" % longdist
    return select_one(query)[0]

def get_id_dist_eole(dist_eole='eole-dev'):
    """
    renvoit l'id de la distribution
    """
    query = """select id from dist_eole where name='%s';""" % dist_eole
    return select_one(query)[0]

def get_branches_from_eoledistrib(dist_eole='', shortdist=False):
    """
    renvoit la(les) branche(s) de compilation git
    """
    if shortdist:
        query = """ select dist_ubuntu.branches from dist_ubuntu where dist_ubuntu.name = '%s'; """ % dist_eole
    else:
        query = """ select dist_ubuntu.branches from dist_ubuntu, dist_eole where dist_ubuntu.name = dist_eole.dist_ubuntu and dist_eole.name = '%s'; """ % dist_eole
    return select_one(query)[0]

def get_dist_branches():
    """
    renvoit les branches stables des différentes distributions eole
    """
    query = """ select distinct dist_ubuntu.branches, dist_ubuntu.eole from dist_ubuntu, dist_eole where dist_ubuntu.name = dist_eole.dist_ubuntu; """
    return select_all(query)

def get_eole_distributions(dist_ubuntu=None):
    """
    liste des distributions Eole existant dans la base
    """
    query = """select name
            from dist_eole """
    if dist_ubuntu is not None:
        query += "where dist_ubuntu LIKE '%s'" % dist_ubuntu
    return select_all(query)

def get_compilable_distributions(dist_ubuntu='feisty'):
    """
    liste des distributions Eole que l'on a droit de compiler
    """
    query = """select name
            from dist_eole
            where dist_ubuntu LIKE '%s'
            and compilable=True""" % dist_ubuntu
    return select_all(query)

def get_ubuntu_distributions():
    """
    liste des distributions Ubuntu existant dans la base
    """
    query = """select dist_ubuntu.name
            from dist_ubuntu"""
    return select_all(query)

def get_ubuntu_from_eole(dist_eole='2.1'):
    """
    récupérer la distrib ubuntu à partir de la version d'Eole
    """
    query = """select dist_ubuntu.name
            from dist_ubuntu
            where dist_ubuntu.eole LIKE '%s'""" % dist_eole
    return select_one(query)[0]

def get_eole_packages_version(dist_ubuntu='feisty', name=None):
    """
    renvoit la liste des paquets disponibles pour une distribution
    """
    query = """select paquets.id, paquets.name, paquets.version
            from paquets, dist_ubuntu
            where dist_ubuntu.name='%s'
            and dist_ubuntu.id=paquets.id_dist_ubuntu""" % dist_ubuntu
    if name != None:
        query += "\nand paquets.name LIKE '%s'" % name
    return select_all(query)

def get_binaires(source):
    """
    renvoit la liste des éventuels paquets binaires
    associés au paquet source
    """
    binaires = []
    query = """select binaire from binaires
               where source LIKE '%s'""" % source
    for bin in select_all(query):
        binaires.append(bin[0])
    return binaires

def create_paquet(paquet, version, svnconfig, dist_ubuntu='feisty'):
    """
    ajout d'un nouveau paquet dans la base de donnée
    """
    id_dist_ubuntu = get_id_dist_ubuntu(dist_ubuntu)
    query = """INSERT INTO paquets (name, version, id_dist_ubuntu)
                VALUES ('%s', '%s', %d);""" % (paquet, version, id_dist_ubuntu)
    execute(query)
    id_paquet = get_id_paquet(paquet, version, dist_ubuntu)
    query = """INSERT INTO paquets_conf (id_paquet, svn_path)
                VALUES (%d, '%s');""" % (id_paquet, svnconfig)
    execute(query)
    return (id_paquet)

def affecte_paquet(id_paquet, id_module):
    """
    affecte un paquet à un module Eole
    """
    query = """INSERT INTO paquets_modules (id_paquet, id_module)
                VALUES (%d, '%d');""" % (id_paquet, id_module)
    execute(query)
    return True

def desaffecte_paquet(id_paquet, id_module):
    """
    désaffecte un paquet à un module Eole
    """
    query = """DELETE FROM paquets_modules
               where id_paquet='%s'
               and id_module='%s';""" % (id_paquet, id_module)
    execute(query)
    return True

def affecte_module(user, id_module):
    """
    affecte un module a un utilisateur
    """
    query = """INSERT INTO utilisateurs
               VALUES ('%s', %d);""" % (user, id_module)
    execute(query)
    return True

def desaffecte_module(user, id_module):
    """
    désaffecte un module a un utilisateur
    """
    query = """DELETE FROM utilisateurs
               where utilisateurs.utilisateur='%s'
               and idmodules='%s';""" % (user, id_module)
    execute(query)
    return True

def create_release(id_paquet, release, long_release, dist_eole='eole-dev'):
    """
    ajout d'une nouvelle release d'un paquet
    """
    # FIXME : attention aux types
    id_dist_eole = get_id_dist_eole(dist_eole)
    query = """INSERT INTO paquets_release (id_paquet, release, long_release, id_dist_eole)
                VALUES (%d, %d, '%s', %d);""" % (id_paquet, release, long_release, id_dist_eole)
    return execute(query)

def add_depends(paquet, dep):
    """
    ajout d'une dépendance
    """
    query = """INSERT INTO dependances (paquet, dependance)
                VALUES ('%s', '%s');""" % (paquet, dep)
    try:
        execute(query)
    except pgsql.IntegrityError:
        return False
    return True

def get_user_modules(user):
    """
    renvoit les accès au module par utilisateur
    """
    query = """ select modules.id, modules.name from modules, utilisateurs where
                modules.id = utilisateurs.idmodules and utilisateurs.utilisateur = '%s'
                order by modules.name;""" % user
    #if select_all(query)[0][0] == 0:
    #    query = """ select modules.id, modules.name from modules; """
    return select_all(query)
    #res = select_one(query)
    #return res[0]

def get_list_user():
    """
    renvoit la liste des utilisateurs
    """
    query = """ select utilisateurs.utilisateur from utilisateurs group by utilisateurs.utilisateur; """
    return select_all(query)

def get_packages():
    """
    renvoit la liste des paquets Eole
    """
    query = """select id, name, version from paquets;"""
    return select_all(query)

def get_package(id_paquet):
    """
    renvoit les informations sur un paquet
    """
    query = """select id, name, version from paquets;"""
    return select_all(query)

def get_packages_from_dist_ubuntu(dist_ubuntu='feisty'):
    """
    renvoit la liste des paquets Eole pour un module
    """

    query = """select distinct paquets.id, paquets.name, paquets.version
               from paquets, dist_ubuntu
               where dist_ubuntu.name='%s'
               and dist_ubuntu.id=paquets.id_dist_ubuntu
               order by paquets.name
               ;""" % (dist_ubuntu)
    return select_all(query)

def get_packages_from_module(id_module, dist_ubuntu='feisty'):
    """
    renvoit la liste des paquets Eole pour un module
    """

    query = """select paquets.id, paquets.name, paquets.version
               from paquets, paquets_modules, dist_ubuntu
               where paquets.id = paquets_modules.id_paquet
               and paquets_modules.id_module = '%d'
               and dist_ubuntu.name='%s'
               and dist_ubuntu.id=paquets.id_dist_ubuntu
               order by paquets.name
               ;""" % (id_module, dist_ubuntu)
    return select_all(query)

def get_affectations(id_paquet):
    """
    renvoit la liste des modules auxquels un paquet est affecté
    """
    res = []
    query = """select id_module
               from paquets_modules
               where id_paquet='%s'""" % id_paquet
    # formattage du résultat
    for mod in select_all(query):
        res.append(mod[0])
    return res

def get_unsatisfied_deps(id_module):
    """
    renvoit la liste des dépendances non satisfaites pour un module
    """
    module_packages = get_packages_from_module(id_module)
    unsatisfied = []
    packages = []
    for package in module_packages:
        packages.append(package[1])
    for package in packages:
        for dep in get_depends(package):
            if dep[0] not in packages:
                unsatisfied.append(dep[0])
    unsatisfied.sort()
    return unsatisfied

def get_depends(package_name):
    """
    renvoit les dépendances d'un paquet
    """
    query = """select dependances.dependance
               from dependances
               where dependances.paquet='%s'""" % package_name
    return select_all(query)

def get_user_pwd(user, pwd):
    pwd = md5.new(pwd).hexdigest()
    query = """select name from utilisateurs where name='%s' and pwd='%s'""" % (user, pwd)
    return select_one(query)

def set_user_pwd(user, pwd):
    pwd = md5.new(pwd).hexdigest()
    query = """INSERT INTO utilisateurs VALUES ('%s', '%s');""" % (user, pwd)
    execute(query)
    return True

def delete_package(id_package):
    """ /!\ suppression d'un paquet /!\ """
    query = """delete from paquets_release
               where id_paquet=%d""" % id_package
    execute(query)
    query = """delete from paquets
               where id=%d""" % id_package
    execute(query)
    return True

def add_user(user):
    """ajout d'un utilisateur"""
    query = """ INSERT INTO utilisateurs(utilisateur) VALUES ('%s'); """ % user
    execute(query)
    return True

def del_user(user):
    """suppression d'un utilisateur"""
    query = """DELETE FROM utilisateurs where utilisateurs.utilisateur = '%s'; """ % utilisateur
    execute(query)
    return True
