# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# eolepacksvn : librairie d'accès à Subversion
#
###########################################################################

from eolepack import configserver
from eolepack import eolepackdb
from os.path import join
import os
import subprocess, tempfile, traceback, git

clone_dir = configserver.CLONE_DIR

def git_command(args, project):
    try:
        if args[0] in ['branch', 'clone', 'pull']:
            cmdargs = ['/usr/bin/git']
        else:
            cmdargs = ['/usr/bin/git', '--work-tree', join(clone_dir, project)]
        cmdargs.extend(args)
        print "lancement commande git (depuis %s) : %s" % (os.getcwd(), " ".join(cmdargs))
        gitcmd = subprocess.Popen(cmdargs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        ret = gitcmd.wait()
        stdoutdata, stderrdata = gitcmd.communicate()
        return ret, stdoutdata, stderrdata
    except:
        traceback.print_exc()
        return None

def git_clone(project):
    """ initialisation du repository git """
    try:
        os.chdir(clone_dir)
        #git_url = join(configserver.GIT_ORIG_PATH, project)
        ret, out, err = git_command(('clone', '-q', configserver.GIT_ORIG_PATH+':'+project), project)
        assert ret == 0
        return join(clone_dir, project)
    except:
        traceback.print_exc()
        return None

def git_pull(project):
    """ mise à jour du repository git """
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('pull', '--all'), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_new_branch(project, new_branch):
    """crée une nouvelle branche"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('branch', new_branch), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_checkout(project, branch):
    """checkout une branche dans le projet"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('checkout', branch), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_checkout_from_tag(project, tag, branch):
    """checkout une branche dans le projet"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('checkout', tag, '-b', branch), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None


def git_track_branch(project, branch, remote_branch):
    """récupère localement une branche distante"""
    try:
        os.chdir(join(clone_dir, project))
        print "***", branch, remote_branch, project
        ret, out, err = git_command(('checkout', '--track', '-b', branch, remote_branch), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_branch(project, remote=True):
    """retourne les branches d'un projet"""
    try:
        #repo = git.Repo(join(clone_dir,project))
        #branches = repo.branches
        os.chdir(join(clone_dir, project))
        if remote:
            cmd_args = ('branch', '-r')
        else:
            cmd_args = ('branch',)
        ret, out, err = git_command(cmd_args, project)
        branches = []
        for branch in out.split('\n'):
            branch = branch.replace('*','').strip()
            if branch != '' and not branch.startswith('origin/HEAD'):
                branches.append(branch)
        return branches
    except:
        traceback.print_exc()
        return None

def git_commit(msg, project):
    """commit dans le projet git"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('commit', '-a', '-m', msg), project)
        assert ret == 0
        return out
    except:
        traceback.print_exc()
        return None

def git_push(project, options = ""):
    """Push des modifications sur le serveur"""
    try:
        os.chdir(join(clone_dir, project))
        cmd_args = ['push']
        if options != "":
            cmd_args.extend(options.split())
        ret, out, err = git_command(tuple(cmd_args), project)
        assert ret == 0
        return out
    except:
        traceback.print_exc()
        return None

def git_log(project, limit):
    """ Affichage du log git """
    try:
        repo = git.Repo(join(clone_dir,project))
        # par défaut, log utilise la branche master : on lui passe sa branche active :-(
        commits = repo.log(commit=repo.active_branch)
        return commits[:limit]
    except:
        traceback.print_exc()
        return None

def git_tag(project):
    """ Affichage du tag git """
    try:
        repo = git.Repo(join(clone_dir,project))
        tags = repo.tags
        return tags
    except:
        traceback.print_exc()
        return None

def git_del_tag(project, tag_name):
    """
    supprime un tag
    """
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('tag', '-d', tag_name), project)
        assert ret == 0
        return out
    except:
        traceback.print_exc()
        return None


def git_buildtag(project, long_release):
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('tag', "debian/%s" % long_release, "-m", "Debian release %s" % long_release),project)
        assert ret == 0
        return out
    except:
        traceback.print_exc()
        return None
