# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# eolepacksvn : librairie d'accès à Subversion
#
###########################################################################

from eolepack import configserver
from eolepack import eolepackdb
from os.path import join
import pysvn

svn_path = configserver.SVN_ORIG_PATH

def svnclient(user=None, password=None):
    """
    déclaration personalisée du client subversion
    """
    client = pysvn.Client()
    # certificat SSL non accepté
    def ssl_server_trust_prompt(trust_dict):
        print "Acceptation du certificat proposé par %s" % trust_dict['realm']
        return True, 0, True
    client.callback_ssl_server_trust_prompt = ssl_server_trust_prompt
    # mot de passe SVN non enregistré
    def login(*args):
        print "Utilisation du compte Subversion %s" % user
        return True, user, password, False
    client.callback_get_login = login
    return client

def _svn_ls(client, url):
    """
    parcours des entrées d'un chemin subversion
    """
    try:
        entry_list = client.ls(url)
    except:
        return []
    lst = []
    for entry in entry_list:
        lst.append(entry.data['name'].split('/')[-1])
    return lst

def _sort_tags(tag1, tag2):
    """
    fonction de tri personnalisée
    """
    num1 = int(tag1.replace('eole', ''))
    num2 = int(tag2.replace('eole', ''))
    # les derniers tags d'abord
    return num2 - num1

def get_svn_tags(client, id_paquet):
    """
    récupération des tags existants pour un paquet
    """
    path = eolepackdb.get_svn_path(id_paquet)
    url = join(svn_path, path, 'tags')
    tags = _svn_ls(client, url)
    tags.sort(_sort_tags)
    return tags

def get_svn_branches(client, id_paquet):
    """
    récupération des branches existantes pour un paquet
    """
    path = eolepackdb.get_svn_path(id_paquet)
    url = join(svn_path, path, 'branches')
    return _svn_ls(client, url)

def svn_path_exists(client, path):
    """
    vérification de la validité d'un chemin SVN
    """
    url = join(svn_path, path)
    if _svn_ls(client, url) != []:
        return True
    return False

