#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# epack.py : client gtk pour la compilation EoleNg
#
###########################################################################

import pygtk, gtk, gtk.glade, gobject
pygtk.require("2.0")
import os, socket, xmlrpclib, base64
from os.path import join
from glob import glob
from eolepack import eolepacksvn
from eolepack.errors import *
from eolepack.epacktools import *
from eolepack.configclient import CENTRAL_RPC_SERV, SVN_ORIG_PATH

class Epack:
    """
    Classe pour l'utilisation de l'interface graphique
    """

    def __init__(self, gladefile, rc_file=None):

        self.centralserv = xmlrpclib.ServerProxy(CENTRAL_RPC_SERV)
        # Chargement du fichier .glade
        if rc_file != None:
            gtk.rc_parse(rc_file)
        self.widgets = gtk.glade.XML(gladefile)
        # variables de l'application
        self.current_module = None
        self.current_package = None
        self.last_package = None
        self.last_svn_status = None
        self.dist_ubuntu = None
        self.current_branch = None
        self.stable_branch = None
        self.dist_eole = None
        self.user = None
        self.passwd = None
        self.session_id = ''
        self.secret_dir = join(os.environ['HOME'], '.epack')
        if not os.path.isdir(self.secret_dir):
            os.makedirs(self.secret_dir)
        ### accesseurs pour les widgets souvent utilisés
        self.mainwindow = self.widgets.get_widget('main_epack')
        self.login = self.widgets.get_widget('login')
        self.module_tree = self.widgets.get_widget('modules')
        self.package_tree = self.widgets.get_widget('packages')
        self.distrib_combo = self.widgets.get_widget('distribution')
        self.svn_dir_combo = self.widgets.get_widget('svn_dir')
        self.dist_ubuntu_combo = self.widgets.get_widget('entry_distrib')
        self.options_compil = self.widgets.get_widget('options_compil')
        self.changelog = self.widgets.get_widget('changelog')
        self.buildlog = self.widgets.get_widget('buildlog')
        self.buildlog_view = self.widgets.get_widget('buildlog_view')
        self.aboutdialog = self.widgets.get_widget('aboutdialog')
        self.changelog_tree = self.widgets.get_widget('changelog_tree')
        self.rc = self.widgets.get_widget('buildrc')
        self.rc_tree = self.widgets.get_widget('rc_treeview')
        self.edition = self.widgets.get_widget('edition')
        self.manage_perm = self.widgets.get_widget('manage_perm')
        self.list_user_combo = self.widgets.get_widget('list_user')
        # par defaut : option trunk
        self.widgets.get_widget('radio_trunk').set_active(True)
        # Initialisation des évenements
        callbacks = {'on_main_epack_delete_event':self.quit,
                     'on_login_delete_event':self.login_quit,
                     'on_login_cancel_clicked':self.login_quit,
                     'on_login_ok_clicked':self.authentification,
                     'on_option_clicked': self.run_option,
                     'on_radio_trunk_clicked': self.mod_svn,
                     'on_radio_tags_clicked': self.mod_svn,
                     'on_radio_branches_clicked': self.mod_svn,
                     'on_combo_branch_changed': self.update_branch_changelog,
                     'on_changelog_button_clicked': self.show_changelog,
                     'on_buildlog_button_clicked': self.show_buildlog,
                     'on_quit_activate': self.quit,
                     'on_aboutmenu_activate': self.about,
                     'on_aboutdialog_close': self.hide_about,
                     'on_about_dialog_response': self.hide_about,
                     'on_close_svn_log_clicked': self.hide_changelog,
                     'on_close_build_log_clicked': self.hide_buildlog,
                     'on_changelog_tree_row_activated': self.show_changelog,
                     'on_changelog_tree_toggle_cursor_row': self.show_changelog,
                     'on_dev_clicked': self.update_dev,
                     'on_rc_clicked': self.update_rc,
                     'on_suppr_rc_packages_clicked':self.suppr_rc_packages,
                     'on_compil_selected_clicked': self.compil_selected,
                     'on_valid_edit_clicked': self.valid_edit,
                     'on_suppr_package_clicked': self.suppr_db_package,
                     'on_add_packages_clicked': self.add_packages,
                     'on_remove_packages_clicked': self.remove_packages,
                     'on_btn_add_perm_clicked': self.add_module_perm,
                     'on_btn_rm_perm_clicked': self.remove_module_perm,
                     'on_btn_valid_perm_clicked': self.valid_perm,
                     # gestion des fenêtres secondaires
                     'on_delete_win': self.hide_window,
                     'on_cancel_clicked': self.cancel_rc,
                     'on_btn_manage_perm_clicked': self.show_manage_perm,
                     'on_list_user_changed': self.update_perm_edition,
                     'on_btn_add_user_clicked': self.add_user,
                     'on_btn_del_user_clicked': self.del_user,
                    }
        self.widgets.signal_autoconnect(callbacks)
        # récupération du compte enregistré
        user, pwd = self._get_registration()
        self.widgets.get_widget('entry_login').set_text(user)
        self.widgets.get_widget('entry_pwd').set_text(pwd)
        self.module_tree.get_selection().connect('changed', self.module_changed)
        self.package_tree.connect('row-activated', self.package_clicked)
        # Initialisation des treeviews et des combos
        self.maj_user_combo = False
        self.init_modules()
        self.init_packages()
        self.init_distributions()
        self.init_todist()
        self.init_svn_dirs()
        self.init_list_user()
        self.init_changelog()
        self.init_rc()
        self.init_module_edition()
        self.init_manage_perm()

    def authentification(self, *args):
        """
        validation du compte
        """
        user = self.widgets.get_widget('entry_login').get_text()
        pwd = self.widgets.get_widget('entry_pwd').get_text()
        try:
            self.session_id, self.user_data = self.centralserv.run_authentif(user, pwd)
        except:
            dial = display_popup(self.login, "Erreur lors de l'authentification sur le serveur de compilation", 'Error')
            dial.run()
            dial.destroy()
            return
        if self.session_id == '':
            dial = display_popup(self.login, "Echec de l'authentification", 'Error')
            dial.run()
            dial.destroy()
        else:
            # finalisation de l'initialisation en fonction des choix de l'utilisateur
            self.user = user
            self.passwd = pwd
            self._register_user()
            self.dist_eole = self.dist_ubuntu_combo.get_child().get_text()
            self.dist_ubuntu = self.centralserv.get_ubuntu_from_eole(self.session_id, self.dist_eole)
            # stockage de la branche par défaut de cette distribution
            self.stable_branch = 'origin/%s' % self.centralserv.get_dist_branch(self.session_id, self.dist_ubuntu)
            self.current_branch = self.stable_branch
            # l'adresse du serveur dépend de la version !
            server = self.centralserv.get_server(self.session_id, self.dist_ubuntu)
            if not server:
                dial = display_popup(self.login, "Serveur indéfini\npour %s" % self.dist_ubuntu, 'Error')
                dial.run()
                dial.destroy()
            else:
                self.update_distributions()
                self.login.destroy()
                self.mainwindow.show()
                self.update_modules()

    def about(self, *args):
        self.aboutdialog.show()

    def hide_about(self, *args):
        self.aboutdialog.hide()
        return True

    def _register_user(self):
        """
        enregistrement du compte local
        """
        crypt = base64.encodestring('%s#%s' % (self.user, self.passwd))
        fp = file(join(self.secret_dir, 'user'), 'w')
        fp.write(crypt)
        fp.close()

    def _get_registration(self):
        user = ''
        pwd = ''
        if os.path.isfile(join(self.secret_dir, 'user')):
            fp = file(join(self.secret_dir, 'user'), 'r')
            crypt = base64.decodestring(fp.read())
            fp.close()
            try:
                user, pwd = crypt.split('#')
            except:
                pass
        return user, pwd

    def init_modules(self):
        """treeview des modules/releases
        colonne 0 = id_module/id_release
        colonne 1 = libelle_mod/libelle_mod+vers_majeure+vers_mineure
        """
        model = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Modules', renderer, text=1)
        self.module_tree.append_column(column)
        self.module_tree.set_model(model)
        self.module_tree.set_headers_visible(True)
        self.module_tree.set_search_column(1)

    def init_module_edition(self):
        """treeview des modules dispo/affectés
        colonne 0 = id_module
        colonne 1 = libelle_module
        """
        trees = { 'tree_available' : 'Modules disponibles',
                  'tree_enable' : 'Modules affectés' }
        for name, label in trees.items():
            model = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING)
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(label, renderer, text=1)
            tree = self.widgets.get_widget(name)
            tree.append_column(column)
            tree.set_model(model)
            tree.set_headers_visible(True)
            tree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

    def init_manage_perm(self):
        """treeview des modules dispo/affectés
        colonne 0 = id_module
        colonne 1 = libelle_module
        """
        trees = { 'tree_perm_available' : 'Modules disponibles',
                  'tree_perm_enable' : 'Modules affectés' }
        for name, label in trees.items():
            model = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING)
            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(label, renderer, text=1)
            tree = self.widgets.get_widget(name)
            tree.append_column(column)
            tree.set_model(model)
            tree.set_headers_visible(True)
            tree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

    def init_packages(self):
        """treeview des packages/modules
        colonne 0 = id_packages
        colonne 1 = libelle_packages
        """
        model = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Paquets', renderer, text=1)
        self.package_tree.append_column(column)
        column = gtk.TreeViewColumn('Version', renderer, text=2)
        self.package_tree.append_column(column)
        self.package_tree.set_model(model)
        self.package_tree.set_headers_visible(True)
        self.package_tree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.package_tree.set_search_column(1)

    def init_distributions(self):
        """combobox des distributions
        dans la fenêtre options
        """
        model = gtk.ListStore(gobject.TYPE_STRING)
        self.distrib_combo.set_model(model)

    def init_todist(self):
        """ combobox pour la recompilation """
        model = gtk.ListStore(gobject.TYPE_STRING)
        self.widgets.get_widget('todist').set_model(model)

    def init_dist_ubuntu(self):
        """combobox des distributions
        dans la fenêtre login
        """
        model = gtk.ListStore(gobject.TYPE_STRING)
        self.dist_ubuntu_combo.set_model(model)

    def init_svn_dirs(self):
        """combobox des repertoires svn
        dans la fenêtre options
        """
        model = gtk.ListStore(gobject.TYPE_STRING)
        self.svn_dir_combo.set_model(model)
        # FIXME : pourquoi celui-là veut cette ligne et pas l'autre ?
        self.svn_dir_combo.set_text_column(0)

    def init_list_user(self):
        """ combobox des users """
        model = gtk.ListStore(gobject.TYPE_STRING)
        self.list_user_combo.set_model(model)
        self.list_user_combo.set_text_column(0)

    def init_changelog(self):
        """treeview des changelog
        colonne 0 = date
        colonne 1 = auteur
        colonne 2 = revision
        colonne 3 = message
        """
        model = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Date', renderer, text=0)
        self.changelog_tree.append_column(column)
        column = gtk.TreeViewColumn('Auteur', renderer, text=1)
        self.changelog_tree.append_column(column)
        column = gtk.TreeViewColumn('Revision', renderer, text=2)
        self.changelog_tree.append_column(column)
        column = gtk.TreeViewColumn('Message', renderer, text=3)
        self.changelog_tree.append_column(column)
        self.changelog_tree.set_model(model)
        self.changelog_tree.set_headers_visible(True)

    def init_rc(self):
        """
        treeview des rc
        colonne 0 = id_paquet
        colonne 1 = non_paquet
        colonne 2 = version
        colonne 3 = distribution
        """
        # sélection multiple
        self.rc_tree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        model = gtk.TreeStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Paquets', renderer, text=1)
        self.rc_tree.append_column(column)
        column = gtk.TreeViewColumn('Version', renderer, text=2)
        self.rc_tree.append_column(column)
        column = gtk.TreeViewColumn('Distribution', renderer, text=3)
        self.rc_tree.append_column(column)
        self.rc_tree.set_model(model)
        self.rc_tree.set_headers_visible(True)

    def update_rc(self, *args):
        self._update_rc('proposed')

    def update_dev(self, *args):
        self._update_rc('dev')

    def _update_rc(self, distro='dev'):
        """remplit la liste des paquets en dev ou rc"""
        # modifie le titre de la fenêtre :)
        version = 'eole-%s-%s' % (self.dist_eole, distro)
        self.widgets.get_widget('rclabel').set_text('Paquets %s' % version)
        # liste déroulante pour la destination
        if distro == 'dev':
            destinations = ['proposed', 'updates', 'security']
            selected = 0
        else:
            destinations = ['updates', 'security']
            selected = 1
        model = self.widgets.get_widget('todist').get_model()
        model.clear()
        for dest in destinations:
            model.append(["eole-%s-%s" % (self.dist_eole, dest)])
        self.widgets.get_widget('todist').set_active(selected)
        # recherche sur le site de mise à jour
        # TODO : Passage de l'url en param
        url = 'http://test-eoleng.ac-dijon.fr/eoleng/%s/' % version
        if self.dist_eole not in ('2.0', '2.1'):
            # nouvelle arborescence
            url += 'source/'
        packages = self.centralserv.list_packages(self.session_id, url)
        model = self.rc_tree.get_model()
        model.clear()
        for package in packages:
            mod_iter = model.insert_before(None, None, None)
            model.set_value(mod_iter, 1, package[0])
            model.set_value(mod_iter, 2, package[1])
            model.set_value(mod_iter, 3, version)
        self.rc.show()

    def update_modules(self):
        """remplit la liste des modules"""
        modules = self.centralserv.get_user_modules(self.session_id, self.user)
        model = self.module_tree.get_model()
        model.clear()
        for module in modules:
            mod_iter = model.insert_before(None, None)
            model.set_value(mod_iter, 0, module[0])
            model.set_value(mod_iter, 1, module[1].capitalize())

    def update_module_edition(self, id_paquet):
        """remplit l'affectation des modules"""
        modules = self.centralserv.get_modules(self.session_id)
        affectations = self.centralserv.get_affectations(self.session_id, id_paquet)
        mod_available = self.widgets.get_widget('tree_available').get_model()
        mod_available.clear()
        mod_enable = self.widgets.get_widget('tree_enable').get_model()
        mod_enable.clear()
        for module in modules:
            if module[0] in affectations:
                mod_iter = mod_enable.insert_before(None, None)
                mod_enable.set_value(mod_iter, 0, module[0])
                mod_enable.set_value(mod_iter, 1, module[1].capitalize())
            else:
                mod_iter = mod_available.insert_before(None, None)
                mod_available.set_value(mod_iter, 0, module[0])
                mod_available.set_value(mod_iter, 1, module[1].capitalize())

    def update_perm_edition(self, user):
        """remplit l'affectation des modules"""
        if self.maj_user_combo:
            # hack pour éviter les appels multiples générés par "on_change"
            return False
        modules = self.centralserv.get_modules(self.session_id)
        user = self.list_user_combo.get_child().get_text()
        affectations = self.centralserv.get_user_modules(self.session_id, user)
        mod_available = self.widgets.get_widget('tree_perm_available').get_model()
        mod_available.clear()
        mod_enable = self.widgets.get_widget('tree_perm_enable').get_model()
        mod_enable.clear()
        for module in modules:
            if module in affectations:
                mod_iter = mod_enable.insert_before(None, None)
                mod_enable.set_value(mod_iter, 0, module[0])
                mod_enable.set_value(mod_iter, 1, module[1].capitalize())
            else:
                mod_iter = mod_available.insert_before(None, None)
                mod_available.set_value(mod_iter, 0, module[0])
                mod_available.set_value(mod_iter, 1, module[1].capitalize())

    def update_packages(self, id_module):
        """remplit la liste des paquets"""
        if id_module == 0:
            # hack pour "tous les paquets
            packages = self.centralserv.get_packages_from_dist_ubuntu(self.session_id, self.dist_ubuntu)
        else:
            packages = self.centralserv.get_packages_from_module(self.session_id, id_module, self.dist_ubuntu)
        model = self.package_tree.get_model()
        model.clear()
        for packages in packages:
            mod_iter = model.insert_before(None, None)
            model.set_value(mod_iter, 0, packages[0])
            model.set_value(mod_iter, 1, packages[1])
            model.set_value(mod_iter, 2, packages[2])

    def update_changelog(self, id_paquet):
        """
        lecture du changelog svn
        """
        local_path = self.centralserv.get_package_name(self.session_id, id_paquet)
        if local_path == '':
            return False, "Paquet non présent dans le dépot GIT"
        str_buffer = self.centralserv.get_log(self.session_id, id_paquet, 20, self.stable_branch)
        model = self.changelog_tree.get_model()
        model.clear()
        self.changelog_tree.set_model(model)
        for log in str_buffer:
            mod_iter = model.insert_before(None, None, None)
            model.set_value(mod_iter, 0, log[0])
            model.set_value(mod_iter, 1, log[1])
            model.set_value(mod_iter, 2, log[2])
            model.set_value(mod_iter, 3, log[3])
        return True, ''

    def update_manage_perm(self):
        """
        mise à jour de la liste des utilisateurs
        """
        self.maj_user_combo = True
        users = self.centralserv.get_list_user(self.session_id)
        model = self.list_user_combo.get_model()
        model.clear()
        if users == []:
            self.list_user_combo.set_sensitive(False)
            self.list_user_combo.child.set_text('')
        else:
            self.list_user_combo.set_sensitive(True)
            for u in users:
                model.append(u)
            self.list_user_combo.set_active(0)
        self.maj_user_combo = False

    def show_manage_perm(self, *args):
        """
        Affichage de la fenetre d'autorisations
        """
        self.update_manage_perm()
        self.update_perm_edition(self.user)
        self.manage_perm.show()

    def show_changelog(self, *args):
        """
        Affichage du changelog SVN d'un paquet
        """
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist == []:
            # selection vide
            dial = display_popup(self.mainwindow, "Vous devez sélectionner\nun paquet !", 'Error')
            dial.run()
            dial.destroy()
            return
        # on ne s'embête pas, on prend le 1er
        path_sel = pathlist[0]
        iter_sel = model.get_iter(path_sel)
        id_paquet = model.get_value(iter_sel, 0)
        package = "%s-%s" % (model.get_value(iter_sel, 1), model.get_value(iter_sel, 2))
        res, msg = self.update_changelog(id_paquet)
        if not res:
            # erreur
            dial = display_popup(self.mainwindow, msg, 'Error')
            dial.run()
            dial.destroy()
            return
        title = 'Changelog GIT du paquet %s (branche : %s)' % (package, self.stable_branch)
        self.widgets.get_widget('label_svn').set_text(title)
        self.changelog.set_size_request(800, 450)
        self.changelog.show()
        #selection.unselect_all()

    def show_buildlog(self, *args):
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        # test sur la sélection
        if pathlist == []:
            dial = display_popup(self.mainwindow, "Vous devez sélectionner\nun paquet !", 'Error')
            dial.run()
            dial.destroy()
            return
        path_sel = pathlist[0]
        iter_sel = model.get_iter(path_sel)

        package = "%s-%s" % (model.get_value(iter_sel, 1), model.get_value(iter_sel, 2))

        string = self.centralserv.get_rapportfile(self.session_id, model.get_value(iter_sel, 1), model.get_value(iter_sel, 2), self.dist_ubuntu)

        # test si le fichier existe
        if string == "":
            dial = display_popup(self.mainwindow, "Fichier de rapport non trouvé! ", 'Error')
            dial.run()
            dial.destroy()
            return
        else:
            title = 'Log compilation du paquet %s' % (package)
            self.widgets.get_widget('label_build').set_text(title)
            textbuffer = self.buildlog_view.get_buffer()
            textbuffer.set_text(string)
            self.buildlog.set_size_request(800, 450)
            self.buildlog.show()

    def hide_changelog(self, *args):
        self.changelog.hide()

    def hide_buildlog(self, *args):
        self.buildlog.hide()

    def update_distributions(self):
        """remplit la liste des distributions"""
        distributions = self.centralserv.get_compilable_distributions(self.session_id, self.dist_ubuntu)
        model = self.distrib_combo.get_model()
        model.clear()
        active = 0
        for num in range(0, len(distributions)):
            distro = distributions[num][0]
            model.append([distro])
            if 'dev' in distro:
                active = num
        self.distrib_combo.set_active(active)

    def update_dist_ubuntu(self):
        """remplit la liste des distributions Ubuntu"""
        distributions = self.centralserv.get_ubuntu_distributions(self.session_id)
        model = self.dist_ubuntu_combo.get_model()
        model.clear()
        for d in distributions:
            model.append([d[0]])
        self.dist_ubuntu_combo.set_active(2)

    def update_svn_dirs(self):
        """remplit la liste des repertoires svn"""
        dirs = []
        if self.current_package == self.last_package: same_package = True
        else: same_package = False
        self.last_package = self.current_package
        if self.widgets.get_widget('radio_branches').get_active():
            if same_package == True and self.last_svn_status == 'branches':
                return
            self.last_svn_status = 'branches'
            for branch in self.centralserv.get_branches(self.session_id, self.current_package):
                # on n'affiche pas la branche HEAD et les branches stables
                if branch not in ['origin/HEAD', 'origin/upstream', self.stable_branch]:
                    dirs.append(branch)

        elif self.widgets.get_widget('radio_tags').get_active():
            if same_package == True and self.last_svn_status == 'tags':
                return
            self.last_svn_status = 'tags'
            dirs = self.centralserv.get_tags(self.session_id, self.current_package)

        else:
            if same_package == True and self.last_svn_status == 'trunk':
                return
            self.last_svn_status = 'trunk'

        model = self.svn_dir_combo.get_model()
        model.clear()

        if dirs == []:
            self.svn_dir_combo.set_sensitive(False)
            self.svn_dir_combo.child.set_text('')
        else:
            self.svn_dir_combo.set_sensitive(True)
            for d in dirs:
                model.append([d])
            self.svn_dir_combo.set_active(0)

    def run_option(self, *args):
        """
        on a cliqué sur "Compiler"
        """
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist == []:
            # selection vide
            dial = display_popup(self.mainwindow, "Vous devez sélectionner\nun paquet !", 'Error')
            dial.run()
            dial.destroy()
            return
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            id_paquet = model.get_value(iter_sel, 0)
            name = model.get_value(iter_sel, 1)
            version = model.get_value(iter_sel, 2)
            self.option(id_paquet, name, version)
        selection.unselect_all()

    def update_branch_changelog(self, *args):
        """
        changement de branche, on récupère le bon changelog
        """
        if self.widgets.get_widget('radio_branches').get_active():
            branch = self.svn_dir_combo.get_child().get_text()
        else:
            branch = self.stable_branch
        if self.current_branch != branch:
            self.current_branch = branch
            self.update_compil_changelog(self.current_package)

    def update_compil_changelog(self, id_paquet):
        # recherche de la branche dans laquelle prendre les logs
        mylog = ""
        local_path = self.centralserv.get_package_name(self.session_id, id_paquet)
        if local_path != '':
            str_buffer = self.centralserv.get_log(self.session_id, id_paquet, 20, self.current_branch)
            for log in str_buffer:
                if log[3].startswith('**'):
                    continue
                elif log[3].startswith('##'):
                    if mylog == "":
                        mylog = "(pas de commit depuis : %s)" % log[3].replace('**', '').replace('##', '')
                    break
                mylog += "%s\n" % log[3]
        buffer = self.widgets.get_widget('changelog_entry').get_buffer()
        buffer.set_text(mylog)

    def option(self, id_paquet, name, version):
        """
        affichage de la fenêtre des options
        """
        title = 'Compiler le paquet %s-%s' % (name, version)
        self.widgets.get_widget('label_compil').set_text(title)
        # compilation depuis trunk par défaut
        self.widgets.get_widget('radio_trunk').set_active(True)
        # maj de la liste des répertoires svn
        self.last_package = self.current_package
        self.current_package = id_paquet
        self.current_name = name
        try:
            self.update_svn_dirs()
        except UnknownConfig:
            dial = display_popup(self.mainwindow, "Pas de configuration connue pour ce paquet !", 'Error')
            res = dial.run()
            dial.destroy()
            return
        # pré-remplissage du changelog
        self.update_compil_changelog(id_paquet)
        self.options_compil.show()
        reponse = self.options_compil.run()
        self.options_compil.hide()
        if reponse != 0:
            return
        # lancement d'une compilation avec options
        tags=branches=revision=sign=cd=depot=None
        if self.widgets.get_widget('radio_branches').get_active():
            branches = self.svn_dir_combo.get_child().get_text()
            if branches == '':
                dial = display_popup(self.mainwindow, "Branche invalide !", 'Error')
                res = dial.run()
                dial.destroy()
                self.option()
                return
        if self.widgets.get_widget('radio_tags').get_active():
            tags = self.svn_dir_combo.get_child().get_text()
            if tags == '':
                dial = display_popup(self.mainwindow, "Tag invalide !", 'Error')
                res = dial.run()
                dial.destroy()
                self.option()
                return
        text = self.widgets.get_widget('changelog_entry').get_buffer()
        start = text.get_start_iter()
        end = text.get_end_iter()
        changelog = text.get_text(start,end)
        distribution = self.distrib_combo.get_child().get_text()
        if changelog == '':
            changelog = None
        else:
            changelog = changelog.replace("\n",";")
        epoch = None

        self.run_compilation(id_paquet, name, version, distribution, tags, branches, revision, epoch, changelog)

    def run_compilation(self, id_paquet, name, version, distribution='eole-dev', tags=None, branches=None, revision=None, epoch=None, changelog=None, affok=True, msg=''):
        """
        lancement effectif de la compilation
        """
        # serialisation de la chaine None pour le xmlrpc
        for var in ['tags','branches','revision','epoch','changelog']:
            if eval(var) == None:
                exec(var+"='None'")

        dial1 = display_popup(self.mainwindow,"Compilation de %s-%s\nVeuillez patienter...\n%s" %
                (name, version , msg), 'Message')
        rundial(dial1)
        dial2 = False
        try:
            res, msg = self.centralserv.run_pack(self.session_id, name, version, distribution, self.user, self.passwd, tags, branches, revision, epoch, changelog, self.dist_ubuntu)
            if res:
                if affok:
                    dial2 = display_popup(self.mainwindow, msg, '')
            else :
                dial2 = display_popup(self.mainwindow, msg, 'Error')
        except socket.error:
            res = False
            self.thread_finished = True
            dial2 = display_popup(self.mainwindow, "Impossible de contacter\nle serveur de compilation !", 'Error')
        dial1.destroy()
        if dial2:
            dial2.run()
            dial2.destroy()
        return res

    def module_changed(self, selected_module, *args):
        """ un module a été sélectionné """
        store, iter_sel = selected_module.get_selected()
        if iter_sel != None:
            self.current_module = store.get_value(iter_sel, 0)
            self.update_packages(self.current_module)

    def package_clicked(self, selected_package, *args):
        """ double click sur un paquet """
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_package = model.get_value(iter_sel, 0)
        package = model.get_value(iter_sel, 1)
        version = model.get_value(iter_sel, 2)
        # reset de la branche courante si changement de paquet
        self.current_branch = self.stable_branch
        try:
            svn_path = join(SVN_ORIG_PATH, self.centralserv.get_svn_path(self.session_id, id_package))
        except:
            svn_path = "chemin GIT inconnu !"
        title = 'Paquet %s-%s\n\n%s\n' % (package, version, svn_path)
        self.widgets.get_widget('label_edition').set_text(title)
        self.update_module_edition(id_package)
        self.edition.show()

    def valid_edit(self, btn):
        """ Edition d'un paquet """
        self.edition.hide()

    def valid_perm(self, btn):
        """ Validation permission """
        self.update_modules()
        self.manage_perm.hide()

    def suppr_db_package(self, btn):
        """ Suppression d'un paquet de la base de données """
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_package = model.get_value(iter_sel, 0)
        package = model.get_value(iter_sel, 1)
        version = model.get_value(iter_sel, 2)
        self.edition.hide()
        dial = display_popup(self.mainwindow,
                "Etes-vous sûr de supprimer le paquet %s-%s ?" % (package, version),
                'Confirm')
        res = dial.run()
        dial.destroy()
        if res != gtk.RESPONSE_YES:
            return
        self.centralserv.delete_db_package(self.session_id, id_package)
        self.update_packages(self.current_module)

    def mod_svn(self, *args):
        try:
            self.update_svn_dirs()
        except UnknownConfig:
            dial = display_popup(self.mainwindow, "Pas de configuration connue pour ce paquet !", 'Error')
            res = dial.run()
            dial.destroy()

    def compil_selected(self, *args):
        """ recompilation de paquets """
        packages = []
        select = self.rc_tree.get_selection()
        model, pathlist = select.get_selected_rows()
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            package_name = model.get_value(iter_sel, 1)
            long_version = model.get_value(iter_sel, 2).split('-eole')
            distribution = model.get_value(iter_sel, 3)
            refresh = True
            if len(long_version) == 2:
                package_version = long_version[0]
                package_release = long_version[1]
                packages.append([package_name, package_version, package_release])
            else:
                print "skipping", package_name
        select.unselect_all()
        if len(packages) > 0:
            self.mass_compil(packages, distribution)

    def suppr_rc_packages(self, *args):
        """ suppression de paquets dans un dépot """
        refresh_needed = False
        select = self.rc_tree.get_selection()
        model, pathlist = select.get_selected_rows()
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            package_name = model.get_value(iter_sel, 1)
            package_version = model.get_value(iter_sel, 2)
            long_version = model.get_value(iter_sel, 2).split('-eole')
            distribution = model.get_value(iter_sel, 3)
            if len(long_version) == 2:
                refresh_needed = True
                dial = display_popup(self.mainwindow,
                """Etes-vous sûr de supprimer le paquet %s-%s
du dépôt %s ?""" % (package_name, package_version, distribution), 'Confirm')
                res = dial.run()
                dial.destroy()
                if res != gtk.RESPONSE_YES:
                    continue
                try:
                    res, msg = self.centralserv.delete_rc_package(self.session_id, package_name, package_version, distribution)
                except socket.error:
                    res = False
                if not res:
                    dial = display_popup(self.mainwindow,
                            """Erreur de suppresion du paquet %s dans %s""" % (package_name, distribution),
                            'Error')
                    dial.run()
                    dial.destroy()
            else:
                print "skipping", package_name
        select.unselect_all()
        if refresh_needed:
            self.rc.hide()
            if distribution.endswith('dev'):
                self._update_rc('dev')
            else:
                self._update_rc('proposed')

    def mass_compil(self, packages, distribution):
        """ recompilation de plusieurs paquets """
        # sauvegarde de la source pour affichage
        if distribution.endswith('dev'):
            distro = 'dev'
        else:
            distro = 'proposed'
        # destination
        new_dist = self.widgets.get_widget('todist').get_child().get_text()
        self.rc.hide()
        total = len(packages)
        encours = 0
        for package in packages:
            encours += 1
            name, version, tag = package
            try:
                id_paquet = self.centralserv.get_id_paquet(self.session_id, name, version, self.dist_ubuntu)
            except UnknownPackage:
                print "Paquet inconnu %s-%s" % (name, version)
                continue
            if self.centralserv.get_svn_path(self.session_id, id_paquet) == '':
                print "Paquet non préset dans GIT %s-%s" % (name, version)
                continue
            log = "Compilation en %s a partir du tag eole%s" % (new_dist, tag)
            #debiantag = '%s/%s-eole%s' % ("debian", version, tag)
            message = "Paquet %d sur %d" % (encours, total)
            res = self.run_compilation(id_paquet, name, version, distribution=new_dist, changelog='', affok=False, msg=message)
            if res:
                # suppression de l'ancien paquet (ancienne distribution)
                try:
                    long_version = '%s-eole%s' % (version, tag)
                    res, msg = self.centralserv.delete_rc_package(self.session_id, name, long_version, distribution)
                except socket.error:
                    res = False
                if not res:
                    dial = display_popup(self.mainwindow,
                            """Erreur de suppresion du paquet %s dans %s""" % (name, distribution),
                            'Error')
                    dial.run()
                    dial.destroy()

        # on raffraichit la page
        self._update_rc(distro)

    def add_packages(self, btn):
        """ affectation de paquets """
        # récup pourrie du paquet encore sélectionné !
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_paquet = model.get_value(iter_sel, 0)
        # modules à affecter
        selection = self.widgets.get_widget('tree_available').get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist != []:
            for path_sel in pathlist:
                iter_sel = model.get_iter(path_sel)
                id_module = model.get_value(iter_sel, 0)
                self.centralserv.affecte_paquet(self.session_id, id_paquet, id_module)
            self.update_module_edition(id_paquet)

    def remove_packages(self, btn):
        """ désaffectation de paquets """
        # récup pourrie du paquet encore sélectionné !
        selection = self.package_tree.get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_paquet = model.get_value(iter_sel, 0)
        # modules à désaffecter
        selection = self.widgets.get_widget('tree_enable').get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist == []:
            # selection vide
            return True
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            id_module = model.get_value(iter_sel, 0)
            self.centralserv.desaffecte_paquet(self.session_id, id_paquet, id_module)
        self.update_module_edition(id_paquet)
        return True

    def add_module_perm(self, btn):
        """ affectation des permissions sur un module """
        # récup pourrie du paquet encore sélectionné !
        selection = self.widgets.get_widget('tree_perm_available').get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_module = model.get_value(iter_sel, 0)
        user = self.list_user_combo.get_child().get_text()
        # modules à affecter
        selection = self.widgets.get_widget('tree_perm_available').get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist == []:
            # selection vide
            return
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            id_module = model.get_value(iter_sel, 0)
            self.centralserv.affecte_module(self.session_id, user, id_module)
        self.update_perm_edition(user)

    def remove_module_perm(self, btn):
        """ désaffectation des permissions sur un module """
        # récup pourrie du paquet encore sélectionné !
        selection = self.widgets.get_widget('tree_perm_enable').get_selection()
        model, pathlist = selection.get_selected_rows()
        iter_sel = model.get_iter(pathlist[0])
        id_paquet = model.get_value(iter_sel, 0)
        user = self.list_user_combo.get_child().get_text()
        # modules à désaffecter
        selection = self.widgets.get_widget('tree_perm_enable').get_selection()
        model, pathlist = selection.get_selected_rows()
        if pathlist == []:
            # selection vide
            return
        for path_sel in pathlist:
            iter_sel = model.get_iter(path_sel)
            id_module = model.get_value(iter_sel, 0)
            self.centralserv.desaffecte_module(self.session_id, user, id_module)
        self.update_perm_edition(user)

    def add_user(self):
        user = self.list_user_combo.get_child().get_text()
        self.centralserv.add_user(self.session_id, user)

    def del_user(self):
        user = self.list_user_combo_get_child().get_text()
        self.centralserv.del_user(self.session_id, user)

    def quit(self, *args):
        """quitte le programme"""
        gtk.main_quit()

    def login_quit(self, *args):
        """quitte le programme"""
        self.login.destroy()
        import sys
        sys.exit()

    def hide_window(self, window, *args):
        """ sauvegarde des fenêtres secondaires """
        window.hide()
        return True

    def cancel_rc(self, *args):
        self.hide_window(self.widgets.get_widget('buildrc'))

def main():
    gladepath = '/usr/share/eolepack'
    if not os.path.isdir(gladepath):
        # pas installé -> fichier local
        gladepath = os.path.dirname(os.path.abspath(__file__)) + '/ui/'
    gladefile = os.path.join(gladepath, 'epack.glade')
    rc_file = os.path.join(gladepath, 'eole.rc')
    epack = Epack(gladefile, rc_file)
    gtk.main()

if __name__ == "__main__":
    main()

