# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# pack.py : librairie pour la génération des paquets EoleNg
#
###########################################################################

from eolepack.configserver import EOLEPACK_FILE, CLONE_DIR
from eolepack.build import Build
from eolepack.errors import *
#from eolepack.cfgparser import Config
from eolepack import eolepackdb
from eolepack import tools
from time import time
from os import chdir, mkdir, system
from os.path import join
import traceback

def pack(paquet, version, distribution, user, passwd, tags=None,
         branches=None, revision=None, epoch=None, changelog=None):
    """
    toutes les étapes pour la création d'un paquet
    """

    dist_ubuntu = eolepackdb.get_ubuntu_from_dist(distribution)
    # récupération des infos dans la base de données
    try:
        id_paquet = eolepackdb.get_id_paquet(paquet, version, dist_ubuntu)
        git_project = eolepackdb.get_svn_path(id_paquet)
    except UnknownPackage:
        return False, "Paquet %s-%s inconnu" % (paquet, version)

    if tags:
        if "-eole" not in tags:
            return False, "Tag non valide : %s" % tags
        revision = 'debian/%s' % tags

    # Instanciation de l'objet Build
    try:
        build = Build(git_project, paquet, branches, distribution, revision)
        chdir(join(CLONE_DIR, git_project))
    except:
        traceback.print_exc()
        return False, "Erreur lors de la récupération des sources du projet"
    autoinc = False
    try:
        # Récupération du dernier numéro de release (HUM, HUM)
        release = file("debian/changelog").read().split()[1].split('eole')[1][0:-1]
        if '~' in release:
            autoinc = True
            release = int(release.split("~")[0])
        else:
            release = int(release) + 1
    except:
        return False, "Erreur lors de la récupération\ndu dernier numéro de release !"
    long_release = "%s-eole%d" % (version, release)
    if epoch:
        long_release = "%s:%s" % (epoch, long_release)
    # édition changelog
    try:
        build.insert_debchange(long_release, user, autoinc)
    except ChangelogNotFound:
        return False, "Fichier changelog non trouvé"
    except DebChangeError:
        return False, "Erreur lors du debchange"
    # compilation du paquet
    try:
        build.debuild(user, passwd)
    except BuildError:
        return False, "Erreur lors de la compilation du paquet\nVeuillez consulter le rapport ##FIXME##"
    ## commit et envoi du changelog et des tags
    try:
        build.push_changes(user, long_release)
    except Exception, msg:
        return False, str(msg)
    ## envoi sur le repository
    res = build.put_deb_repository(repository='eole')
    if res != 0:
        return False, "Erreur d'envoi des paquets sur le dépot eole"

    # Récupération du numéro de release après compilation
    snapshot_info = ""
    release = file("debian/changelog").read().split()[1].split('eole')[1][0:-1]
    if '~' in release:
        snapshot_info = release[release.index("~"):]
        release = int(release.split("~")[0])
    else:
        release = int(release)
    if snapshot_info:
        success_msg = "Paquet snapshot compilé avec succès:\n%s-%s%s" % (paquet, long_release, snapshot_info)
    else:
        success_msg = "Paquet stable compilé avec succès:\n%s-%s" % (paquet, long_release)
    return True, success_msg
