#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Serveur xmlrpc pour la compilation des paquets EoleNg
#
###########################################################################

from twisted.web import xmlrpc, server
from twisted.internet import reactor, defer
from OpenSSL import SSL
from eolepack.pack import pack
from eolepack.delete import delete_package
import time, xmlrpclib, os
from glob import glob
from eolepack import eolepackdb, eolepacksvn, eolerc, ldapauth, tools, eolepackgit, build
from eolepack.configserver import COMPIL_PATH, KEY_FILE, CERT_FILE, SSO_SERV
from datetime import datetime

class ServerXMLRPC(xmlrpc.XMLRPC):

    sso_serv = xmlrpclib.ServerProxy('https://%s:8443/xmlrpc' % SSO_SERV)

    def render_POST(self, request):
        """
        fonction render_POST modifiée pour authentifier les appels
        """
        request.content.seek(0, 0)
        request.setHeader("content-type", "text/xml")
        try:
            args, functionPath = xmlrpclib.loads(request.content.read())
            # vérification de l'authentification. Un argument supplémentaire (id de session)
            # doit être donné au début de chaque appel
            if len(args) >= 1 and functionPath != 'run_authentif':
                sso_session = args[0]
                if self.sso_serv.validate_session(sso_session):
                    user = self.sso_serv.get_user_info(sso_session)
                    print "SSO Reported username : %s"  % user
                else:
                    raise "Invalid SSO Session"
                # on supprime la session sso des arguments d'appel à la fonction RPC
                args = args[1:]
        except Exception, e:
            f = xmlrpc.Fault(self.FAILURE, "Can't deserialize input: %s" % (e,))
            self._cbRender(f, request)
        else:
            try:
                function = self._getFunction(functionPath)
            except xmlrpc.Fault, f:
                self._cbRender(f, request)
            else:
                defer.maybeDeferred(function, *args).addErrback(
                    self._ebRender
                ).addCallback(
                    self._cbRender, request
                )
        return server.NOT_DONE_YET

    def xmlrpc_delete_package(self, paquet, version, dist_eole):
        return delete_package(paquet, version, dist_eole)

    def xmlrpc_run_authentif(self, user, pwd):
        # création d'une session SSO
        return self.sso_serv.authenticate(user, pwd)
        # return ldapauth.authenticate(user, pwd)

    def xmlrpc_get_ubuntu_from_eole(self, dist_eole):
        return eolepackdb.get_ubuntu_from_eole(dist_eole)

    def xmlrpc_get_server(self, dist_ubuntu):
        #return eolepackdb.get_server(dist_ubuntu)
        return 'http://192.168.10.143:7081'

    def xmlrpc_get_modules(self):
        return eolepackdb.get_modules()

    def xmlrpc_get_user_modules(self, user):
        return eolepackdb.get_user_modules(user)

    def xmlrpc_get_affectations(self, id_paquet):
        return eolepackdb.get_affectations(id_paquet)

    def xmlrpc_get_packages_from_dist_ubuntu(self, dist_ubuntu):
        return eolepackdb.get_packages_from_dist_ubuntu(dist_ubuntu)

    def xmlrpc_get_packages_from_module(self, id_module, dist_ubuntu):
        return eolepackdb.get_packages_from_module(id_module, dist_ubuntu)

    def xmlrpc_get_compilable_distributions(self, dist_ubuntu):
        return eolepackdb.get_compilable_distributions(dist_ubuntu)

    def xmlrpc_get_ubuntu_distributions(self):
        return eolepackdb.get_ubuntu_distributions()

    def xmlrpc_get_dist_branches(self):
        return eolepackdb.get_dist_branches()

    def xmlrpc_get_svn_path(self, id_paquet):
        return eolepackdb.get_svn_path(id_paquet)

    def xmlrpc_get_package_name(self, id_paquet):
        return eolepackdb.get_package_name(id_paquet)

    def xmlrpc_delete_package(self, id_package):
        return eolepackdb.delete_package(id_package)

    def xmlrpc_get_id_paquet(self, name, version, dist_ubuntu):
        return eolepackdb.get_id_paquet(name, version, dist_ubuntu)

    def xmlrpc_affecte_paquet(self, id_paquet, id_module):
        return eolepackdb.affecte_paquet(id_paquet, id_module)

    def xmlrpc_desaffecte_paquet(self, id_paquet, id_module):
        return eolepackdb.desaffecte_paquet(id_paquet, id_module)

    def xmlrpc_affecte_module(self, user, id_module):
        return eolepackdb.affecte_module(user, id_module)

    def xmlrpc_desaffecte_module(self, user, id_module):
        return eolepackdb.desaffecte_module(user, id_module)

    def xmlrpc_list_packages(self, url):
        return eolerc.list_packages(url)

    def xmlrpc_replace_str(self, changelog):
        return tools.replace_str(changelog)

    def xmlrpc_get_branches(self, id_paquet):
        git_project = eolepackdb.get_svn_path(id_paquet)
        return eolepackgit.git_branch(git_project)

    def xmlrpc_get_dist_branch(self, shortdist):
        return eolepackdb.get_branches_from_eoledistrib(shortdist, True)

    def xmlrpc_get_tags(self, id_paquet):
        git_project = eolepackdb.get_svn_path(id_paquet)
        str_buffer = eolepackgit.git_tag(git_project)
        data = []
        for tag in str_buffer:
            data.append((tag.name))
        return data

    def xmlrpc_get_log(self, id_paquet, limit, branch):
        git_project = eolepackdb.get_svn_path(id_paquet)
        project = eolepackdb.get_package_name(id_paquet)
        build.Build(git_project, project, branch)
        str_buffer = eolepackgit.git_log(git_project, limit=limit)
        data = []
        for log in str_buffer:
            #data.append((str(datetime.fromtimestamp(int(log["date"]))), str(log["author"]), str(log["revision"].number), str(log["message"])))
            data.append((str(datetime.fromtimestamp(time.mktime(log.committed_date))), str(log.author), str(log.id_abbrev), str(log.summary)))
        return data

    def xmlrpc_get_server(self, dist_ubuntu):
        return eolepackdb.get_server(dist_ubuntu)

    def xmlrpc_get_rapportfile(self, name, version, dist_ubuntu):
        self.compiler = xmlrpclib.ServerProxy(eolepackdb.get_server(dist_ubuntu))
        string = self.compiler.get_rapportfile(name, version)
        return string

    def xmlrpc_run_pack(self, name, version, distribution, user, passwd, tags, branches, revision, epoch, changelog, dist_ubuntu):
        self.compiler = xmlrpclib.ServerProxy(eolepackdb.get_server(dist_ubuntu))
        string = self.compiler.pack(name, version, distribution, user, passwd, tags, branches, revision, epoch, changelog)
        return string

    def xmlrpc_get_list_user(self):
        return eolepackdb.get_list_user()

    def xmlrpc_add_user(self, user):
        return eolepackdb.add_user(user)

    def xmlrpc_del_user(self, user):
        return eolepackdb.del_user(user)

class ServerContextFactory:
    """
    Factory permettant de créer un contexte SSL
    à utiliser pour les serveurs twisted (xmlrpc ...)
    """

    def __init__(self, certfile='', keyfile=''):
        self.keyfile = None
        if certfile != '':
            self.certfile = certfile
        else:
            self.certfile = cert_file
        if keyfile != '':
            self.keyfile = keyfile
        else:
            self.keyfile = key_file

    def getContext(self):
        """Initialisation du contexte SSL et chargement des certificats
        """
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.use_certificate_file(self.certfile)
        try:
            ctx.use_privatekey_file(self.certfile)
        except:
            if self.keyfile is not None:
                alt_keyfile = self.keyfile
            else:
                # impossible de charger la clé depuis le fichier du certificat on regarde si il n'y a pas un fichier .key associé
                alt_keyfile = os.path.splitext(self.certfile)[0] + '.key'
            ctx.use_privatekey_file(alt_keyfile)
        return ctx

if __name__ == '__main__':
    from twisted.python import log
    from twisted.internet import reactor
    import sys
    print
    print time.ctime()
    print "démarrage du serveur central de rêquete sur le port 7080"
    print
    log.startLogging(sys.stdout)
    r = ServerXMLRPC()
    factory = ServerContextFactory(CERT_FILE, KEY_FILE)
    reactor.listenSSL(7080, server.Site(r), factory)
    reactor.run()
